package com.miya.manning.test.flow.order;

import com.miya.manning.framework.util.*;
import com.miya.manning.vo.conditions.Condition;
import com.miya.manning.vo.scripts.Script;
import com.miya.manning.vo.testcase.TestCase;
import org.apache.log4j.Logger;
import org.testng.Assert;
import org.testng.annotations.*;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.stream.Collectors;

/**
 * @ProjectName: manning_test
 * @Package: com.miya.manning.test.flow.order.Delivery
 * @ClassName: Mainprocess
 * @Description: 万宁物流模块测试
 * @Author: 熊军奕
 * @CreateDate: 2022/5/23 下午22:38
 * @UpdateUser: 熊军奕
 * @UpdateDate: 2022/5/23 下午22:38
 */
public class Express {

    static Logger logger = Logger.getLogger(Express.class);
    /*private String baseUrl = PropertiesHandle.readValue(StringUtil.brandurl);
    private String apiprefixUrl = StringUtil.brand_apiprefix;*/
    //private String testUrl = "web-front/coupon/get-page";
    //private String url_start = baseUrl + apiprefixUrl ;
    private String url_start = "https://cs2-uat.mannings.com.cn/";
    private String script_path = System.getProperty("user.dir") + File.separator + "target" + File.separator + "classes"
            + File.separator + "testcase" + File.separator + "Scripts" + File.separator + "Script.xls";
    private String condition_path = System.getProperty("user.dir") + File.separator + "target" + File.separator + "classes"
            + File.separator + "testcase" + File.separator + "Conditions" + File.separator + "Condition.xls";
    private String path = System.getProperty("user.dir") + File.separator + "target" + File.separator + "classes" + File.separator + "testcase" + File.separator + "Express.xls";
    //private String path = "D:\\autotest\\test_liucheng\\test_liucheng\\huihua_saas1.xls";
    private Map<String,String> res_map = new TreeMap<>();
    private static Map<String,Integer> int_var_map = new HashMap<>();
    private static Map<String,String> string_var_map = new HashMap<>();
    private static List<Script> Scripts = null;
    private static List<Condition> Conditions = null;


    @DataProvider(name = "ex")
    public Object[][] Parameter(){

        /*return new Object[][]{
                {1,10, "200", "null", null},//查询券列表
                {2,10, "200", "null", null},//查询券列表第二页
                //{ "200", "null", null},//
        };*/
        Object[][] testcases = null;
        try {
            Scripts = TestCaseUtil.GetScripts(script_path);
            Conditions = TestCaseUtil.GetConditions(condition_path);
            testcases = TestCaseUtil.GetMain(path);
        } catch (RuntimeException e) {
            logger.error("error:" , e );

        }
        return testcases;
    }

    @BeforeClass
    public void setUp() {
        logger.info("分页查询券接口测试开始1！");
    }

    @AfterClass
    public void tearDown() {

        logger.info("分页查询券接口测试结束！");
        for (Map.Entry<String, String> entry : res_map.entrySet())
            logger.info(entry.getKey() + ":" + entry.getValue());
    }

    @BeforeMethod
    public void before(){
    }


    @Test(dataProvider = "ex")
    public void Express( TestCase testCase) throws Exception {
        //前置语句最先执行
        String pre_sql = testCase.getPre_sql();
        String databasename = testCase.getDatabasename();
        //1.处理前置语句,赋值给变量或者直接执行
        TestCaseUtil.deal_pre_sql(pre_sql, string_var_map, int_var_map, res_map , databasename);
        //检查是否有控制器,判断是否执行该用例
        String condition_name = testCase.getCondition_name();
        if(!condition_name.isEmpty()){
            List<Condition> execConditionList = Conditions.stream().filter(x -> x.getCondition_name().equals(testCase.getCondition_name())).collect(Collectors.toList());
            boolean flag = ToolUtil.ExecuteCondition(execConditionList, Scripts ,testCase ,string_var_map, int_var_map, res_map);
            if(flag){
                //控制器返回结果为true时,说明已经执行过用例了或者无需执行用例,这里就不用再执行一次
                logger.info("取消当前组用例执行,用例名称:" + testCase.getStep_desc() );
                return;
            }
        }
        String script_name = testCase.getScript_name();
        if(!script_name.isEmpty()){
            List<Script> execScriptList = Scripts.stream().filter(x -> x.getScript_name().equals(testCase.getScript_name())).collect(Collectors.toList());
            ToolUtil.ExecuteScript(execScriptList);
        }

        //用例执行部分
        //2.设置请求头
        Map<String, String> httpheadermap = new HashMap<>();
        if("C".equals(testCase.getPlatform())){
            httpheadermap  = HttpHeaderBuilt.httpHeadBuiltManningsC(string_var_map,int_var_map);
        }
        if("B".equals(testCase.getPlatform())){
            httpheadermap = HttpHeaderBuilt.httpHeadBuiltManningsB(res_map);
        }

        logger.info("以下是请求头信息:");
        for(Map.Entry<String,String> entry: httpheadermap.entrySet()){
            logger.info(entry.getKey() + ":" + entry.getValue());
        }
        //4.调用前替换params中可能存在的变量
        String params = testCase.getParams();
        logger.info("替换前的参数为:" + params);
        String new_params = ParamsBuilt.params_replace(params, int_var_map, string_var_map ,res_map);
        logger.info("替换后的参数为:" + new_params);
        //5.拼接url,调用并获取接口返回
        StringBuffer url = new StringBuffer(url_start);
        StringBuilder builder = new StringBuilder();
        url.append(testCase.getInterface_name());//先拼接一部分
        String new_url = ParamsBuilt.params_replace(url.toString(), int_var_map, string_var_map, res_map);//替换url中可能存在的变量
        StringBuffer new_url_bf = new StringBuffer(new_url);
        RequestUtil.doRequest(testCase, new_params, new_url_bf, httpheadermap, logger, builder);
        //6.收集响应信息，并按规则添加至map中
        JsonTransfer.AnalysisJson(builder.toString(),res_map);
        //7.执行检查语句,和期望值相同
        String check_sql = testCase.getCheck_sql();
        Assert.assertEquals(TestCaseUtil.do_check_sql(check_sql,string_var_map,int_var_map,res_map,databasename), true);
        //8.校验期望值
        String expect = testCase.getExpect();
        Assert.assertEquals(TestCaseUtil.check(expect, int_var_map,string_var_map,res_map), true);








        /*if (code == "200") {
            Response response = new Response(final_res.getString("code"), final_res.getString("msg"), final_res.getJSONObject("data"));
            JSONObject final_res_data = (JSONObject) response.getData();
            Assert.assertEquals(response.getCode(), code);
            Assert.assertEquals(response.getMsg(), msg);


            String manufacturer_id =PropertiesHandle.readValue(StringUtil.httpHeader, "manufacturerId");
            List<CouponConfigPO> couponGetPagelist = CouponSql.couponGetPage(manufacturer_id,pageNum,pageSize);
            for (int i = 0; i< couponGetPagelist.size(); i++) {
                Assert.assertEquals(final_res_data.getJSONArray("list").getJSONObject(i).getString("manufacturerId"),couponGetPagelist.get(i).getManufacturerId());
                Assert.assertEquals(final_res_data.getJSONArray("list").getJSONObject(i).getString("picUrl"),couponGetPagelist.get(i).getPicUrl());
                Assert.assertEquals(final_res_data.getJSONArray("list").getJSONObject(i).getString("couponScene"),couponGetPagelist.get(i).getCouponScene());
                Assert.assertEquals(final_res_data.getJSONArray("list").getJSONObject(i).getString("couponName"),couponGetPagelist.get(i).getCouponName());
                Assert.assertEquals(final_res_data.getJSONArray("list").getJSONObject(i).getString("couponRights"),couponGetPagelist.get(i).getCouponRights());
                Assert.assertEquals(final_res_data.getJSONArray("list").getJSONObject(i).getString("couponType"),couponGetPagelist.get(i).getCouponType());
                Assert.assertEquals(final_res_data.getJSONArray("list").getJSONObject(i).getString("couponBatch"),couponGetPagelist.get(i).getCouponBatch());
                Assert.assertEquals(final_res_data.getJSONArray("list").getJSONObject(i).getString("instructions"),couponGetPagelist.get(i).getInstructions());
                Assert.assertEquals(final_res_data.getJSONArray("list").getJSONObject(i).getString("state"),couponGetPagelist.get(i).getState());
                Assert.assertEquals(final_res_data.getJSONArray("list").getJSONObject(i).getInt("costScore"),couponGetPagelist.get(i).getCostScore());
                Assert.assertEquals(final_res_data.getJSONArray("list").getJSONObject(i).getInt("userDailyGetLimit"),couponGetPagelist.get(i).getUserDailyGetLimit());
            }


        } else {
            Response response = new Response(final_res.getString("code"), final_res.getString("msg"), final_res.getString("data"));
            Assert.assertEquals(response.getCode(), code);
            Assert.assertEquals(response.getMsg(), msg);
            Assert.assertEquals(response.getData(), data);
        }*/
    }



}
